import random
import cocos
import pyglet
from pyglet.window import key, mouse
from Context import Velocity
from vec2d import Vec2d


class Actor(cocos.cocosnode.CocosNode):
    def __init__(self, context, build_data):
        super(Actor, self).__init__()
        
        self.x = build_data.pos.x
        self.y = build_data.pos.y
        self.height = build_data.height

        self.sprites = [] 
        if build_data.anim is not None:
            imgseq = []
            for i in range(1,build_data.anim.frames+1):
                img = pyglet.resource.image( "%s%02d.png" % (build_data.image, i) )
                imgseq.append(img)
                                             
            anim = pyglet.image.Animation.from_image_sequence( imgseq, build_data.anim.delay, build_data.anim.loop )
            self.sprites.append( cocos.sprite.Sprite(anim) )
            #self.sprite.color = (0, 200, 200)
        elif build_data.image is not None:
            self.sprites.append( cocos.sprite.Sprite( build_data.image ) )
            
        for s in self.sprites:
            s.scale = build_data.scale
            s.rotation = build_data.rotation
            context.rootnode.add( s, build_data.z )
        
        self.velocity = Velocity(0,0,0)
        self.speed = 100.0
        
        
    def destroy(self, context):
        for s in self.sprites:
            context.rootnode.remove(s)
            
    def update(self, context):
        self.x      += self.velocity.x      * self.speed * context.dt
        self.y      += self.velocity.y      * self.speed * context.dt
        self.y      += self.velocity.height * context.dt
        self.height += self.velocity.height * context.dt
        
        
        for s in self.sprites:
            s.x = self.x
            s.y = self.y
        
        #for s in self.sprites:
            #if s in self.children:
            #    self.remove(s)
            #else:
                #self.add(s, context.getActorZ(self) )
    
    def getRealY(self):
        return self.y - self.height
    
    