import math
import random
import cocos
import pyglet
from pyglet.window import key, mouse
from vec2d import Vec2d
from cocos.actions.interval_actions import RandomDelay
from cocos.actions.instant_actions import CallFunc
import Actor
import Wolf

class Bird(Actor.Actor):
    
    leg_lengths = (21, 25, 34, 46, 57)
    brainoffset = Vec2d(11, 20)
    beekoffset  = Vec2d(23, 15)
    wingoffset  = Vec2d(-10, -5)
    
    def __init__(self, context, build_data):
        super(Bird, self).__init__(context, build_data)
                
        if build_data.genes == None:
            self.genes = {}
            self.genes["jumping"] = random.randint(1,3)
            self.genes["flying"] = random.randint(1,3)
            self.genes["nomminess"] = random.randint(1,3)
            self.genes["reproducibility"] = random.randint(1,3)
            self.genes["cleverness"] = random.randint(1,3)
        else:
            self.genes = build_data.genes
               
        br  = pyglet.image.load( "../data/brainsizes01.png" )
        wi  = pyglet.image.load( "../data/wingsizes01.png" )
        be  = pyglet.image.load( "../data/beeksizes01.png" )
        rf = pyglet.image.load( "../data/rightfootsizes01.png" )
        rf = pyglet.image.load( "../data/rightfootsizes01.png" )
        lf = pyglet.image.load( "../data/leftfootsizes01.png" )
        self.image = {}
        self.image["bird"] = pyglet.image.load( "../data/bird01.png" )        
        self.image["rightfoot"] = range(5)
        self.image["leftfoot"]  = range(5)
        self.image["brain"]  = range(5)
        self.image["beek"]  = range(5)
        self.image["wing"]  = range(5)
        
                
        for i in range(5):
            self.image["rightfoot"][i] = rf.get_region(i*rf.width/5, 0, rf.width/5, self.leg_lengths[i])
            self.image["leftfoot"][i]  = lf.get_region(i*lf.width/5, 0, lf.width/5, self.leg_lengths[i])
            self.image["leftfoot"][i]  = lf.get_region(i*lf.width/5, 0, lf.width/5, self.leg_lengths[i])
            self.image["brain"][i]     = br.get_region(i*br.width/5, 0, br.width/5, br.height)
            self.image["beek"][i]      = be.get_region(i*be.width/5, 0, be.width/5, be.height)
            self.image["wing"][i]      = wi.get_region(i*wi.width/5, 0, wi.width/5, wi.height)

        leg_index = int( (self.genes["jumping"]    - 1 ) / 2 )
        br_index  = int( (self.genes["reproducibility"] - 1 ) / 2 )
        be_index  = int( (self.genes["nomminess"]  - 1 ) / 2 )
        wi_index  = int( (self.genes["flying"]     - 1 ) / 2 )
        
        self.bs = cocos.sprite.Sprite( self.image["bird"] )
        self.rl = cocos.sprite.Sprite( self.image["rightfoot"][leg_index] )
        self.ll = cocos.sprite.Sprite( self.image["leftfoot"][leg_index] )
        self.br = cocos.sprite.Sprite( self.image["brain"][br_index] )
        self.be = cocos.sprite.Sprite( self.image["beek"][be_index] )
        self.wi = cocos.sprite.Sprite( self.image["wing"][wi_index] )
        
        self.bs_offset = self.leg_lengths[leg_index] + self.bs.height/2 - 15
        #self.bs.y = self.bs_offset
        self.rl.y += self.rl.height/2
        self.ll.y += self.rl.height/2
        self.rl.x =10
        
        self.br.x = self.brainoffset.x
        self.br.y = self.brainoffset.y
        self.be.x = self.beekoffset.x
        self.be.y = self.beekoffset.y
        self.wi.x = -20
        self.wi.y = 0
        
        
        self.birdnode = cocos.cocosnode.CocosNode()
        context.rootnode.add( self.birdnode, z = context.getActorZ(self) )
        self.birdbody = cocos.cocosnode.CocosNode()
        self.birdbody.scale = 1.0 + self.genes["nomminess"]*0.05
        self.birdnode.add( self.birdbody, z=0.5 )
        self.birdnode.add( self.rl, z=0.4 )
        self.birdnode.add( self.ll, z=0.4 )
        
        self.birdbody.y=self.bs_offset
        self.birdbody.add( self.bs, z=0.5 )
        self.birdbody.add( self.br, z=0.6 )
        self.birdbody.add( self.be, z=0.6 )
        
        self.wingnode = cocos.cocosnode.CocosNode()
        self.wingnode.x = 10
        self.wingnode.y = self.wingoffset.y
        self.wingnode.add( self.wi )
        self.birdbody.add(self.wingnode, z=0.6)
        #self.sprites.append( self.bs )
        #self.sprites.append( self.br )
        #self.sprites.append( self.be )
        #self.sprites.append( self.wi )
        #self.sprites.append( self.rl )
        #self.sprites.append( self.ll )
        
        
        self.zl = cocos.text.Label('lols', width=500, color = (0,0,0,255), position = (0,90), multiline=True)        
        #self.birdnode.add(self.zl)
        
        self.shadow = cocos.sprite.Sprite( "shadow.png" )
        self.shadow.x = 5
        self.birdnode.add(self.shadow)

        self.isTouchingGround = True
        self.speed = 30
        self.t = 4*random.random()
        
        if build_data.state is None:
            self.state = "Walking"
        else:
            self.state = build_data.state
            
        self.foott = random.random()
        
        self.prevy_z_recalc = 0
        
        
        s = cocos.sprite.Sprite("readyfor.png")
        s.y = 60
        s.x = -20
        self.bs.add( s, z = 0.7, name = "ready" )
        self.disableMateing()
    
    def disableMateing(self):
        self.readyToMate = False
        delay = 11 - self.genes["reproducibility"]
        self.bs.get("ready").visible = False
        self.do( RandomDelay(delay-1, delay+1) + CallFunc(self.enableMateing))
        
    def enableMateing(self):
        delay = self.genes["reproducibility"] + 3
                    
        self.readyToMate = True
        self.bs.get("ready").visible = True
        self.do( RandomDelay(delay-1, delay+1) + CallFunc(self.disableMateing))
    
    
    def destroy(self, context):
        context.rootnode.remove(self.birdnode)
        
    def update(self, context):
        super(Bird,self).update(context)
        
        if self.state == "Hatching":
            self.birdnode.x=self.x
            self.birdnode.y=self.y
            return
        
        normalized_height_100 = 1.0 - (100.0 - min(100.0,self.height)) * 0.01
        #self.zl.element.text = self.state
        
        #if self.birdbody in self.children:
        #    self.remove(self.birdbody)
        
        if abs(self.prevy_z_recalc - self.getRealY()) > 15:
            self.prevy_z_recalc = self.getRealY()
            context.rootnode.remove(self.birdnode)
            context.rootnode.add(self.birdnode, z = context.getActorZ(self))
            self.birdnode.scale = 1.0 - 0.25 * (self.getRealY() - 100.0) / 300.0


        
        
        if not self.isTouchingGround:
            if self.height < 0:
                #print "% 4d :: % 4d :: % 4d  :: % 4d " \
                #%(self.height, self.y, self.velocity.height,self.genes["jump"])
                self.y -= self.height
                self.height = 0
                self.isTouchingGround = True
                self.velocity.height = 0 
                self.velocity.x = random.randint(0,1)*2 - 1
                self.velocity.y = random.randint(0,1) - 0.5
                self.state = "Walking"
            else:
                
                flying = 1.0 - (10-self.genes["flying"]) * 0.1
                self.wingnode.rotation = 10*math.sin(self.foott * 60 * flying*flying) * normalized_height_100
                
                if self.velocity.height > 0:
                    self.velocity.height -= context.gravity * context.dt
                else:
                    self.velocity.height -= context.gravity * context.dt * (1.05-flying)
                    
            self.shadow.y = - self.height + 5
            
        if 0 < self.height < 100:
            self.shadow.opacity = (1.0-normalized_height_100) * 255
            self.shadow.scale = (1.0-normalized_height_100)
            
        
        if self.isTouchingGround:            
            self.t -= context.dt
            if self.t < 0 and not self.state == "Avoiding":    
                self.jump()
                self.t = 1+3*random.random()
                                
            if self.x < 250:
                self.velocity.x = 1
            elif self.x > 950:
                self.velocity.x = -1
            
            if self.getRealY() < 120:
                self.velocity.y = 0.3
            elif self.getRealY() > 390:
                self.velocity.y = -0.3 
                
            """
            if random.randint(1,10) < self.genes["cleverness"]:
                dist, wolf = context.level.findClosestActor(self, Wolf.Wolf)
                maxdist = self.genes["cleverness"]*self.genes["cleverness"]+50*50
                if wolf is not None and dist < maxdist:
                    if wolf.getRealY() > self.getRealY():
                        self.velocity.y = -2
                    else:
                        self.velocity.y = 2
                    self.state = "Avoiding"
                else:
                    self.state = "Walking"
            """
                
        
        self.rl.x = 5  + math.cos(self.foott * 3) * 6 * (1.0- normalized_height_100)
        self.ll.x = 5 + math.sin(self.foott * 4) * 6 * (1.0- normalized_height_100)
        self.rl.rotation =  8*math.cos(self.foott * 5) * (1.0- normalized_height_100)
        self.ll.rotation =  8*math.sin(self.foott * 4) * (1.0- normalized_height_100)
        self.birdbody.rotation = 10*math.cos(self.foott * 4) * (1.0- normalized_height_100)
        self.birdbody.y = self.bs_offset + (math.sin(self.foott * 15) * 2-2) * (1.0- normalized_height_100)
        
        self.foott += context.dt
        self.birdnode.x=self.x
        self.birdnode.y=self.y
        
    def jump(self):
        self.velocity.y = 0
        self.velocity.height = self.genes["jumping"] * 75
        self.isTouchingGround = False
        self.state = "Jumping"
        