import math
import random
import cocos
import pyglet
from pyglet.window import key, mouse
from vec2d import Vec2d

import Actor
import Bird
import FeatherCloud
from Context import ActorBuildData

class Wolf(Actor.Actor):
    def __init__(self, context, build_data):
        super(Wolf, self).__init__(context, build_data)
                
        self.velocity.x = 3
        #self.velocity.y = 1
        
        self.state = "Approaching"
        
        fl  = pyglet.image.load( "../data/wolffrontlegs01.png" )
        hl  = pyglet.image.load( "../data/wolfhindlegs01.png" )
        ta  = pyglet.image.load( "../data/wolftail.png" )

        self.image = {}
        self.image["frontleg"] = range(4)
        self.image["hindleg"]  = range(4)
        self.image["tail"]  = range(3)
        
        for i in range(4):
            self.image["frontleg"][i] = fl.get_region(i*fl.width/4, 0, fl.width/4, fl.height)
            self.image["hindleg"][i]  = hl.get_region(i*hl.width/4, 0, hl.width/4, hl.height)

        for i in range(3):
            self.image["tail"][i] = ta.get_region(i*ta.width/3, 0, ta.width/3, ta.height)


        self.animt = random.random() * 0.2
        
        
        self.wolfnode = cocos.cocosnode.CocosNode()
        context.rootnode.add( self.wolfnode, z = context.getActorZ(self))
        
        self.z = context.getActorZ(self)
        
        self.head = cocos.sprite.Sprite( "wolfhead01.png" )
        self.body = cocos.sprite.Sprite( "wolf01.png" )
        self.head.x = 50
        self.head.y = 10
        self.wolfnode.add(self.head, z=self.z + 0.1)
        self.wolfnode.add(self.body, z=self.z)
        
        self.fls = cocos.sprite.Sprite( self.image["frontleg"][random.randint(0,3)] )
        self.hls = cocos.sprite.Sprite( self.image["hindleg"][random.randint(0,3)] )
        self.fls2 = cocos.sprite.Sprite( self.image["frontleg"][random.randint(0,3)] )
        self.hls2 = cocos.sprite.Sprite( self.image["hindleg"][random.randint(0,3)] )
        self.tas = cocos.sprite.Sprite( self.image["tail"][random.randint(0,2)] )
        self.wolfnode.add(self.fls)
        self.wolfnode.add(self.hls)
        self.wolfnode.add(self.fls2)
        self.wolfnode.add(self.hls2)
        self.body.add(self.tas)
        
        
        
        self.zl = cocos.text.Label('lols', width=500, color = (255,0,0,255), position = (0,90), multiline=True)        
        #self.add(self.zl)
        
        
        imgseq = []
        for i in range(1,3):
            img = pyglet.resource.image( "smoke%02d.png" % ( i) )
            imgseq.append(img)
                                         
        #anim = pyglet.image.Animation.from_image_sequence( imgseq, 0.3, True )
        #self.explosion = cocos.sprite.Sprite(anim)
        #self.explosion.visible = False
        #context.rootnode.add(self.explosion, z = context.getActorZ(self) + 0.1)
        
        self.shadow = cocos.sprite.Sprite( "wolfshadow.png" )
        self.shadow.x = 5
        self.wolfnode.add(self.shadow)
        

        self.t = 0.0
        self.animt2 = random.random() * 5
        self.opt = random.randint(1,10)

    def update(self, context):
        
        super(Wolf,self).update(context)
                
        #self.zl.element.text = "%s  %f :: %f" % (self.state, self.height, self.y)
        
        #self.explosion.x = self.x
        #self.explosion.y = self.y
        
        self.anim(context)
        
        
        if self.state == "Jumping":
            self.velocity.height -= context.dt * 850
            if self.height < 0.0:
                self.state = "Hunting"
                self.velocity.height = 0.0
                self.velocity.x = 1
                
                if self.x > 1300:
                    context.actors_to_remove.append(self)
        
        if self.state == "Approaching":
            if self.x > -100:
                self.velocity.height = 500
                self.state = "Jumping"
                context.audio.playSound("wolfhowl")
        
        
        self.opt += 1
        if self.opt > 10: 
            self.opt = 0
            if self.state == "Hunting":
                dist, bird = context.level.findClosestActor( self, Bird.Bird)
                    
                if bird is not None and dist < 1200:
                    if bird.height < 20 and bird.state != "Hatching":
                        self.state = "Nomming"
                        #self.explosion.visible = True
                        context.actors_to_remove.append(bird)
                        self.velocity.x = 0
                        context.audio.playSound("wolfnom")
                        
                        build_data = ActorBuildData()
                        build_data.pos = Vec2d(bird.x,bird.y)
                        context.level.createActor( FeatherCloud.FeatherCloud, context, build_data )
                        
                if self.x > 950:
                    self.velocity.x = 4
                    self.velocity.height = 500
                    self.state = "Jumping"
        
        if self.state == "Nomming":
            self.t += context.dt
            if self.t > 2.0:
                self.t = 0.0
                self.state = "Hunting"
                #self.explosion.visible = False
                self.velocity.x = 1
                 
                 
                 
                 
    def anim(self, context):

        self.animt += context.dt
        self.animt2 += context.dt
        
        self.wolfnode.x = self.x
        self.wolfnode.y = self.y + 30
        

        self.shadow.y = - self.height - 40
        
        self.head.y = 10.0 + math.sin(self.animt2 * 15.0) * 5.0
        self.body.x = math.cos(self.animt2 * 7.0) * 4.0
        self.body.y = math.sin(self.animt2 * 11.0) * 3.0
        
        if 0 < self.height < 100:
            normalized_height_100 = 1.0 - (100.0 - min(100.0,self.height)) * 0.01
            self.shadow.opacity = (1.0-normalized_height_100) * 255
            self.shadow.scale = (1.0-normalized_height_100)
        
        if self.animt > 0.2:
            self.animt = 0.0
            self.wolfnode.remove(self.fls)
            self.wolfnode.remove(self.hls)
            self.wolfnode.remove(self.fls2)
            self.wolfnode.remove(self.hls2)
            self.body.remove(self.tas)
            self.fls = cocos.sprite.Sprite( self.image["frontleg"][random.randint(0,3)] )
            self.hls = cocos.sprite.Sprite( self.image["hindleg"][random.randint(0,3)] )
            self.fls2 = cocos.sprite.Sprite( self.image["frontleg"][random.randint(0,3)] )
            self.hls2 = cocos.sprite.Sprite( self.image["hindleg"][random.randint(0,3)] )
            self.tas = cocos.sprite.Sprite( self.image["tail"][random.randint(0,2)] )
            self.fls.x = 30
            self.hls.x = -30
            self.fls2.x = 20
            self.hls2.x = -20
            self.tas.x = -60
            self.fls.y = -30
            self.hls.y = -30
            self.fls2.y = -30
            self.hls2.y = -30
            self.tas.y = 20
            self.wolfnode.add(self.fls, z=self.z-0.1)
            self.wolfnode.add(self.hls, z=self.z-0.1)
            self.wolfnode.add(self.fls2, z=self.z-0.1)
            self.wolfnode.add(self.hls2, z=self.z-0.1)
            self.body.add(self.tas, z=self.z+0.1)
            
            