import cocos

from cocos.scenes.transitions import *
from cocos.actions import *

from pyglet.window import key, mouse

class InputLayer( cocos.layer.base_layers.Layer ):

    is_event_handler = True
        
    def __init__( self ):
        super( InputLayer, self ).__init__()
        
        self.keys_new = set()
        self.keys_old = set()
        
        self.buttons_new = set()
        self.buttons_old = set()
        self.mousepos = ( 0, 0 )
        self.mousepos_rel = ( 0, 0 )

               
    def update( self ):
        self.mousepos_rel = ( 0, 0 )
        self.keys_old = set()
        self.buttons_old = set()
        
        for key in self.keys_new:
            self.keys_old.add( key )
            
        for button in self.buttons_new:
            self.buttons_old.add( button )
            
            
    def is_key_down( self, key ):
        return key in self.keys_new
        
    def is_key_just_pressed( self, key ):
        return key in self.keys_new and not key in self.keys_old
        

    def is_button_down( self, button ):
        return button in self.buttons_new

    def is_button_just_clicked( self, button ):
        return button in self.buttons_new and not button in self.buttons_old

        
    ### Callbacks
    def on_key_press( self, key, modifier ):
        self.keys_new.add( key )
        
    def on_key_release( self, key, modifier ):
        if key in self.keys_new:
            self.keys_new.remove( key )
        
        
    def on_mouse_motion( self, x, y, dx, dy ):
        self.mousepos_rel = dx, dy
        self.mousepos = cocos.director.director.get_virtual_coordinates (x, y)
        
    def on_mouse_drag( self, x, y, dx, dy, buttons, modifiers ):
        self.mousepos_rel = dx, dy
        self.mousepos = cocos.director.director.get_virtual_coordinates (x, y)
        
    def on_mouse_press( self, x, y, buttons, modifier ):
        self.buttons_new.add( buttons )
        
    def on_mouse_release( self, x, y, button, modifier ):
        self.buttons_new.remove( button )
        
    