import math
import random

import cocos
from vec2d import Vec2d

from Context import ActorBuildData, Animation, Z
from Actors.Actor import Actor
from Actors.Bird import Bird
from Actors.Wolf import Wolf
from Actors.Egg import Egg



class Level( cocos.cocosnode.CocosNode ):
    def __init__(self):
        super(Level, self).__init__()
        
        self.actors = []
        self.money = 200
        self.startTexts = []
        self.labels = []
        self.labeli = 0
        
    def initLabels(self, context):
        y = 440
        x = 390
        c=(255,255,255,255)
        
        self.startTexts.append("")
        for t in self.startTexts:
            y -= 20
            l = cocos.text.Label( t, color = c, position = (x,y), font_name="Arial", font_size=14)
            l.opacity = 0
            self.labels.append(l)
            context.rootnode.add(l, z = Z.HUD)
            

        donelabel =  cocos.text.Label( "Press space to start playing!", color = c, position = (390,250), font_name="Arial", font_size=14)
        self.labels.append(donelabel)
        context.rootnode.add(donelabel, z = Z.HUD)
        
        self.briefingbg = cocos.sprite.Sprite( "briefingBG.png" )
        self.briefingbg.x = 600
        self.briefingbg.y = 350
        self.briefingbg.opacity = 220
        context.rootnode.add(self.briefingbg, z = Z.HUD - 0.1)
           
    def start(self,context):
        #print "start", self.labels
        for l in self.labels:
            context.rootnode.remove(l)
        context.rootnode.remove(self.briefingbg)
            
    #def updateBriefing(self, context):
    
    def update(self, context):
        for a in self.actors:
            a.update(context)
            
        for a in context.actors_to_remove:
            a.destroy(context)
            self.actors.remove(a)
            self.remove(a)
            
        for a in context.actors_to_add:
            self.actors.append(a)
            self.add(a)
            
        context.actors_to_remove = []
        context.actors_to_add = []
        
        # labels
        #for i in range(len(self.labels))
        if self.labeli < len(self.startTexts)-1:
            self.labels[self.labeli].opacity += context.dt * 300
            self.labels[self.labeli+1].opacity += context.dt * 200
            if self.labels[self.labeli].opacity >= 254:
                self.labels[self.labeli].opacity = 255
                self.labeli += 1
    
    
    def findClosestActorByPos(self, other, actorType):
        mindist = 100000
        minactor = None
        for a in self.actors:
            if actorType is not None and not isinstance(a, actorType):
                continue
            
            dist = (a.x - other.x)**2 + (a.y - other.y)**2
            if dist < mindist:
                mindist = dist
                minactor = a
                
        return mindist, minactor
    
    
    
    def findClosestActor(self, other, actorType):
        mindist = 100000
        minactor = None
       # return mindist, minactor
        for a in self.actors:
            if actorType is not None and not isinstance(a, actorType):
                continue
            
            dist = (a.x - other.x)**2 + (a.getRealY() - other.getRealY())**2
            if dist < mindist:
                mindist = dist
                minactor = a
                
        return mindist, minactor
    
    def createActor(self, type, context, build_data):
        actor = type( context, build_data )
        self.add(actor)
        self.actors.append(actor)
        return actor
    
    
    def getNextLevel(self):
        return None
    