import math
import random

import cocos
from vec2d import Vec2d

from Context import ActorBuildData, Animation, Z
from Actors.Actor import Actor
from Actors.Bird import Bird
from Actors.Wolf import Wolf
from Actors.Egg import Egg
from Actors.TutorialActor import TutorialActor 

import Level
import Level02


class Level01(Level.Level):
    def __init__(self, context):
        super(Level01, self).__init__()
        
        self.startTexts = []

        self.startTexts.append( "Hi, welcome to my farm!" )
        self.startTexts.append( "I heard you've got some real bird breeding skills.")
        self.startTexts.append( "Now, before I can hire you, I've got to test you.")
        self.startTexts.append( "The first thing I want you to do is to get this")
        self.startTexts.append( "here farm of mine up and running.")
        self.startTexts.append( "I'll supply you with some initial money,  ")
        self.startTexts.append( "and when you've doubled it, I know")
        self.startTexts.append( "you're my kind of breeder!")
        
        self.initLabels(context)
        
        
        self.money = 400
        
        build_data = ActorBuildData()
        
        build_data.image = "farm01.png"
        build_data.pos = Vec2d(1200/2,720/2)
        build_data.z = Z.BG
        
        actor = Actor( context, build_data )
        self.add(actor)
        self.actors.append(actor)
        
        for x in range(4):
            build_data.pos = Vec2d(50+random.randint(0,600), 100 + random.randint(0,200))
            build_data.image = None
            self.createActor(Bird, context, build_data)
        
    def start(self,context):
        super(Level01, self).start(context)
        
        build_data = ActorBuildData()
        build_data.pos = Vec2d(0,0)
        build_data.data["time"] = None
        
        build_data.data["next"] = None
        build_data.data["text"] = "Good luck!"
        tut = self.createActor(TutorialActor, context, build_data)
        
        build_data.data["next"] = tut
        build_data.data["text"] = "The recipe is simple: Slaughter the not-so-nommy ones, breed on the rest, until they are supernommy. By then you will reach a steady flow of income. "
        tut = self.createActor(TutorialActor, context, build_data)
        
        build_data.data["next"] = tut
        build_data.data["text"] = "Breeding will create new birds that have genes similiar, but different, genes from its parents. "
        tut = self.createActor(TutorialActor, context, build_data)
        
        build_data.data["next"] = tut
        build_data.data["text"] = "Now for the fun part: breeding! When a bird wants some lovin', you can select it. To breed, select two birds, and then click the bouncy hearts at the top."
        tut = self.createActor(TutorialActor, context, build_data)
        
        build_data.data["next"] = tut
        build_data.data["text"] = "You will see stars below some of your birds. The ones with the stars under them, are the birds that rank highest in the genes you have clicked on."
        tut = self.createActor(TutorialActor, context, build_data)
        
        build_data.data["next"] = tut
        build_data.data["text"] = "Therefore, you should breed really nommy chickens! Do this by clicking on the NOMMINESS gene icon in the upper left."
        tut = self.createActor(TutorialActor, context, build_data)
        
        
        build_data.data["next"] = tut
        build_data.data["text"] = "As mentioned, you get more money if the bird goes OM NOM NOM in the customers mouth."
        tut = self.createActor(TutorialActor, context, build_data)
        
        build_data.data["next"] = tut
        build_data.data["text"] = "You get money by slaughtering your birds (the meat is sold). To get money from a bird, click the AXE in the top right corner, then click on the bird. Try it, but keep at least two!"
        tut = self.createActor(TutorialActor, context, build_data)
        
        
        build_data.data["next"] = tut
        build_data.data["text"] = "You will lose my every couple of seconds for feeding and keeping the birds - having more birds around will cost you more!"
        tut = self.createActor(TutorialActor, context, build_data)
        
        
        build_data.data["next"] = tut
        build_data.data["text"] = "Ok, lets get cracking. The goal of this level is to get a lot of money. You can see the money in the top right corner."
        tut = self.createActor(TutorialActor, context, build_data)
        
        
        
        #build_data.data["next"] = tut
        #build_data.data["text"] = "The fifth one is CLEVERNESS. The better the smarty gene, the more likely it will avoid the wolf. Observe a bird's thoughts when a wolf approaches to see it in action."
        #tut = self.createActor(TutorialActor, context, build_data)
        
        build_data.data["next"] = tut
        build_data.data["text"] = "The fourth one is REPRODUCIBILITY. The better the love gene, the more often it will want to mate. Look at when a bird thinks of love to know when you can select it for mating."
        tut = self.createActor(TutorialActor, context, build_data)
        
        build_data.data["next"] = tut
        build_data.data["text"] = "The third one is NOMMINESS. The better the nom gene, the more tasty the bird is, and you will get more money when you slaughter it. Look at a bird's size and beek to see if it's really nommy!"
        tut = self.createActor(TutorialActor, context, build_data)
        
        build_data.data["next"] = tut
        build_data.data["text"] = "The second one is FLYING. The better the flying gene, the longer the bird will take to fall to the ground. Look at a bird's wings to see if it's got good flying genes."
        tut = self.createActor(TutorialActor, context, build_data)
                
        build_data.data["next"] = tut
        build_data.data["text"] = "The first one is JUMPING. The better the jumping gene, the higher the bird can jump. Look on a bird's legs to see if it's got good jumping genes."
        tut = self.createActor(TutorialActor, context, build_data)
        
        build_data.data["next"] = tut
        build_data.data["time"] = 2
        build_data.data["text"] = "The first thing you must know as a bird breeder is to learn how to control their genes. You can see the genes in the top left corner. "
        self.createActor(TutorialActor, context, build_data)
        
        
        for x in range(0):
            build_data.pos = Vec2d(-400 - random.randint(0,1250), 150 + random.randint(0,200))
            #build_data.image = "wolf"
            #build_data.anim = Animation()
            self.createActor(Wolf, context, build_data)
        
        
        for x in range(0):
            build_data.pos = Vec2d(200 + x * 100, 100 + random.randint(0,250))
            #build_data.image = "wolf"
            #build_data.anim = Animation()
            self.createActor(Egg, context, build_data)
            
    
    
    def isWon(self, context):
        return self.money > 800
            
    def isLost(self, context):
        return self.money < 0
    
    def getNextLevel(self):
        return Level02.Level02
    