import cocos

from cocos.scenes.transitions import *
from cocos.actions import *

from pyglet.window import key, mouse

from InputLayer import InputLayer
import Context
from MatingLayer import MatingLayer
from SlaughterLayer import SlaughterLayer

import GameOverScene
import Audio

    
class OutdoorsScene( cocos.scene.Scene ):


    def __init__( self, level ):
        super( OutdoorsScene, self ).__init__()
        self.input_layer = InputLayer()
        self.add( self.input_layer )
        #self.do( Repeat( CallFunc( self.update ) ) )
        

        self.updateContext = Context.Context(0.01, self.input_layer)
        self.rootnode = cocos.cocosnode.CocosNode()
        self.updateContext.rootnode = self.rootnode
        self.updateContext.audio = Audio.Audio()
        
        
        self.levelClass = level
        self.level = self.levelClass( self.updateContext )
        self.level.add( self.rootnode)
        self.add( self.level )
        
        self.updateContext.level = self.level

        self.state = "Briefing"
        
        
    def update( self ):
        if self.state == "Playing":
            self.matingLayer.update(self.updateContext)
            self.slaughterLayer.update(self.updateContext)
            
        self.level.update(self.updateContext)
        
        if self.input_layer.is_key_just_pressed(key.SPACE) and self.state == "Briefing":
            self.level.start(self.updateContext)
            
            self.matingLayer = MatingLayer(self.updateContext)
            self.slaughterLayer = SlaughterLayer( self.updateContext, self.matingLayer )
            
            self.add( self.slaughterLayer )
            self.add( self.matingLayer )
            self.state = "Playing"
        
        self.input_layer.update()
        
        if self.state == "Playing":
            if self.level.isWon(self.updateContext):
                self.state = "Transitioning"
                self.remove(self.input_layer)
                self.remove(self.matingLayer)
                self.remove(self.slaughterLayer)
                go_scene = GameOverScene.GameOverScene( self.levelClass, self.level, won=True )
                tr = ZoomTransition(go_scene, duration=1)
                cocos.director.director.replace( tr )
                
            if self.level.isLost(self.updateContext):
                self.state = "Transitioning"
                self.remove(self.input_layer)
                self.remove(self.matingLayer)
                self.remove(self.slaughterLayer)
                go_scene = GameOverScene.GameOverScene( self.levelClass, self.level, won=False )
                tr = ZoomTransition(go_scene, duration=1)
                cocos.director.director.replace( tr )
        
        
    def on_enter( self ):
        super( OutdoorsScene, self ).on_enter()
        self.do( Repeat( CallFunc( self.update ) ) )
        
        