import math
import random
import cocos

from cocos.scenes.transitions import *
from cocos.actions import *

from pyglet.window import key, mouse

from InputLayer import InputLayer
import Context
from Actors.Egg import Egg
from Actors.Bird import Bird
from Actors.FeatherCloud import FeatherCloud

from Context import ActorBuildData, Animation, Z

from vec2d import Vec2d

import random


class SlaughterLayer( cocos.layer.base_layers.Layer ):
    
    is_event_handler = True
    
    def __init__( self, context, matinglayer ):
        super( SlaughterLayer, self ).__init__()

        self.mating = matinglayer
        self.root = context.rootnode
                
        self.hudimg = cocos.sprite.Sprite( "moneyHUD.png" )
        self.hudimg.x = 1000
        self.hudimg.y = 650
        self.root.add(self.hudimg, z = Z.HUD + 0.001)
        
        c=(255,255,200,255)
        self.moneylabel = cocos.text.Label( color = c, position = (0,0), bold=True, font_name="Arial", font_size=30, anchor_x="center")
        self.moneylabel.x = 990
        self.moneylabel.y = 645
        
        
        
        self.axeroot = cocos.cocosnode.CocosNode()
        self.axeroot.x = 1050
        self.axeroot.y = 620
        self.axeimg = cocos.sprite.Sprite( "icon_butcher.png" )
        self.axeimg.x = -32
        self.axeimg.y = -32
        self.axeroot.add(self.axeimg)
        self.root.add(self.axeroot, z = Z.HUD + 0.001)
            
        self.root.add(self.moneylabel, z=Z.HUD + 0.1)
        self.context = context
        
        self.animt = 0.0
        self.t = 0.0
        self.moneyt = 0.0
        self.gotmoneyt = 0.0
        
        self.axeHot = False
        self.axeActive = False
        
        
    def on_mouse_motion( self, x, y, dx, dy ):
        
        mpos = Vec2d( cocos.director.director.get_virtual_coordinates (x, y) )
        axepos = Vec2d(1050, 620)
        
        if mpos.get_dist_sqrd(axepos) < 150*150:
            self.axeHot = True
        else:
            self.axeHot = False
        
        
    def on_mouse_press( self, x, y, buttons, modifier ):
        if self.axeActive:
            self.axeroot.x = 1150
            self.axeroot.y = 600
            #print self.mating.hoverActor
            if self.mating.hoverActor is not None and (self.mating.hoverActor.state == "Walking" or self.mating.hoverActor.state == "Jumping"):
                birdtokill = self.mating.hoverActor
                self.context.actors_to_remove.append(birdtokill)
                
                toearn = int( 10 + birdtokill.genes["nomminess"] * 10)
                self.context.level.money += toearn
                self.mating.deselect()
                self.gotmoneyt = 1.0
                
                self.context.audio.playSound( "earnmoney" )
                
                self.mating.hoverActor = None
                
                build_data = ActorBuildData()
                build_data.pos = Vec2d(birdtokill.x,birdtokill.y)
                self.context.level.createActor( FeatherCloud, self.context, build_data)
                
                
        
        if self.axeHot:
            self.axeActive = not self.axeActive
        
        
        
    def update(self, context):
        self.moneylabel.element.text = str(context.level.money)
        
        if self.gotmoneyt > 0.0:
            self.gotmoneyt -= context.dt*2
            self.hudimg.scale = 1.0 + 0.6 * self.gotmoneyt
        
        if self.axeActive:
            self.axeroot.x = self.mating.mousepos[0] + 40
            self.axeroot.y = self.mating.mousepos[1] + 20
        else:
            self.axeroot.x = 1150
            self.axeroot.y = 600
        
        if context.level.money < 50:
            self.moneyt += context.dt
            color = 0.5 + math.sin(self.moneyt * 3) * 0.5 
            self.hudimg.color = (200 + 55*color, 150,150)
        else:
            self.hudimg.color = (255, 255, 255)
        
        self.t += context.dt
        if self.t > 4.0:
            self.t -= 4.0
            moneytolose = len([a for a in context.level.actors if isinstance(a, Bird)])
            context.level.money -= moneytolose
        
        if self.axeActive and self.mating.hoverActor is not None:
            thesin = 0.5 + math.sin(self.animt * 10) * 0.5 
            self.axeroot.rotation = 100 - 90 * thesin*thesin*thesin*thesin*thesin*thesin
        elif not self.axeActive:
            self.axeroot.rotation = 100
        

        if self.axeHot:
            self.axeimg.opacity =255*( 0.5 + math.sin(self.animt * 15) * 0.5)
        else:
            self.axeimg.opacity =255
            
        self.animt += context.dt
    
    
    
    
    